package ch.psi.daq.rest.queries;

import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;

import org.apache.commons.lang3.builder.ReflectionToStringBuilder;

import ch.psi.daq.cassandra.reader.Ordering;
import ch.psi.daq.hazelcast.common.query.Query;
import ch.psi.daq.hazelcast.common.query.ValueAggregation;
import ch.psi.daq.hazelcast.common.query.bin.BinIntervalCalculator;
import ch.psi.daq.rest.model.Aggregation;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonSubTypes;
import com.fasterxml.jackson.annotation.JsonSubTypes.Type;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.sun.istack.internal.NotNull;

/**
 * 
 * @author zellweger_c
 *
 */
@JsonTypeInfo(
      use = JsonTypeInfo.Id.NAME,
      include = JsonTypeInfo.As.PROPERTY,
      property = "queryType")
@JsonSubTypes(
{
      @Type(value = PulseRangeQuery.class, name = "pulserange"),
      @Type(value = TimeRangeQuery.class, name = "timerange"),
})
public abstract class AbstractQuery implements Query {

   @NotNull
   private boolean isOrderDescending;

   @NotNull
   private List<String> channels;

   @NotNull
   private LinkedHashSet<String> fields;

   @NotNull
   private List<Aggregation> aggregations;

   private ValueAggregation valueAggregation;

   private boolean aggregateChannels;

   /**
    * 
    * @param isOrderDescending whether to add a 'orderBy' clause into the database query
    * @param channelIds all the channelIds (channel names) we want to query
    * @param fields the fields (who map to fields in the DB) we are interested in returning to the
    *        client, needs to be in insertion order (hence the {@link LinkedHashSet} type)
    */
   @JsonCreator
   public AbstractQuery(
         // note that those annotations are needed for the polymorphic
         // mapping to work correctly
         @JsonProperty(value = "isOrderDescending") boolean isOrderDescending,
         @JsonProperty(value = "channels") List<String> channelIds,
         @JsonProperty(value = "fields") LinkedHashSet<String> fields,
         @JsonProperty(value = "aggregateChannels") boolean aggregateChannels,
         @JsonProperty(value = "aggregationType") ValueAggregation valueAggregation,
         @JsonProperty(value = "aggregations") List<Aggregation> aggregations) {

      this.isOrderDescending = isOrderDescending;
      this.aggregateChannels = aggregateChannels;
      this.valueAggregation = valueAggregation;

      if (channelIds == null || fields == null) {
         throw new IllegalArgumentException("sourceIds and/or fields cannot be null.");
      }

      this.channels = channelIds;
      this.fields = fields;
      this.aggregations = aggregations;
   }
   
   /**
    * {@inheritDoc}
    */
   @Override
   public List<String> getChannels() {
      return Collections.unmodifiableList(channels);
   }

   /**
    * {@inheritDoc}
    */
   @Override
   public Ordering getOrdering() {
      if (isOrderDescending) {
         return Ordering.DESC;
      }
      return Ordering.ASC;
   }
   
   /**
    * {@inheritDoc}
    */
   @Override
   public ValueAggregation getValueAggregation() {
      return valueAggregation;
   }
   
   /**
    * {@inheritDoc}
    */
   @Override
   public boolean aggregateChannels() {
      return isAggregateChannels();
   }
   
   @Override
   public BinIntervalCalculator getBinIntervalCalculator() {
      // TODO Auto-generated method stub
      return null;
   }
   
   
   public boolean isAggregateChannels() {
      return aggregateChannels;
   }

   public boolean isOrderDescending() {
      return isOrderDescending;
   }

   public LinkedHashSet<String> getFields() {
      return fields;
   }

   public List<Aggregation> getAggregations() {
      return aggregations;
   }
   
   @Override
   public String toString() {
      return ReflectionToStringBuilder.reflectionToString(this);
   }

}
